# ***************************************************************************
# Copyright 2017-2018 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************
import json
import nsx_logging
from cli.utils.command_utils import invoke_command
from cli.utils.node_utils import get_node_type
from cli.utils.constants import NODE_TYPE_HOST_CENTOSKVM
from cli.utils.constants import NODE_TYPE_HOST_SLESKVM
from cli.utils.constants import NODE_TYPE_HOST_RHELKVM
from cli.utils.constants import NODE_TYPE_HOST_UBUNTUKVM
from cli.utils.constants import NODE_TYPE_HOST_WIN
from cli.utils.constants import NODE_TYPE_BAREMETAL_UBUNTU_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_SLES_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_OEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_WINDOWS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_CONTAINER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_CONTAINER
from cli.utils.location_utils import FilePaths
from cli.utils.error_code import ErrorCode as ec
from cli.commands.cli_command_provider import CliCommandProvider
from cli.commands.host_shared.firewall import DFWVifArgument

lg = nsx_logging.getLogger("cli.commands.host_kvm.firewall")

#
# DFW related commands
#

_FIREWALL_CLI_KVM = {
    NODE_TYPE_HOST_SLESKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_HOST_CENTOSKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_HOST_RHELKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_HOST_UBUNTUKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_HOST_WIN: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_BAREMETAL_UBUNTU_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_BAREMETAL_RHEL_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_BAREMETAL_CENTOS_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_BAREMETAL_SLES_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_BAREMETAL_OEL_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_BAREMETAL_RHEL_CONTAINER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_BAREMETAL_CENTOS_CONTAINER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    NODE_TYPE_BAREMETAL_WINDOWS_SERVER: {
        'bin': FilePaths.OVS_APPCTL_PATH,
        'socket': FilePaths.NSXA_CTL_SOCKET
        },
    }

def has_firewall_cli_kvm(node_type):
   return (node_type == NODE_TYPE_HOST_SLESKVM or
           node_type == NODE_TYPE_HOST_CENTOSKVM or
           node_type == NODE_TYPE_HOST_RHELKVM or
           node_type == NODE_TYPE_HOST_UBUNTUKVM or
           node_type == NODE_TYPE_BAREMETAL_UBUNTU_SERVER or
           node_type == NODE_TYPE_BAREMETAL_RHEL_SERVER or
           node_type == NODE_TYPE_BAREMETAL_CENTOS_SERVER or
           node_type == NODE_TYPE_BAREMETAL_SLES_SERVER or
           node_type == NODE_TYPE_BAREMETAL_OEL_SERVER or
           node_type == NODE_TYPE_BAREMETAL_WINDOWS_SERVER or
           node_type == NODE_TYPE_BAREMETAL_RHEL_CONTAINER or
           node_type == NODE_TYPE_BAREMETAL_CENTOS_CONTAINER or
           node_type == NODE_TYPE_HOST_WIN)

def invoke_firewall_command(cmd_list):
    node_type = get_node_type()
    if has_firewall_cli_kvm(node_type):
        cmd = [_FIREWALL_CLI_KVM[node_type]['bin'], "-t", _FIREWALL_CLI_KVM[node_type]['socket']]
        cmd.extend(cmd_list)
        rc, out, err = invoke_command(cmd)
        out2 = { "results" : "%s" % out }
        return (rc, json.dumps(out2), err)
    else:
        lg.error("node type %s does not have firewall CLI", node_type,
                 error_code=ec.INVALID_ARGUMENT)
        return (-1, None, "invalid command")

def get_firewall_cli_output(command):
    cmd_list = command
    rc, out, err = invoke_firewall_command(cmd_list)
    if rc != 0:
        lg.error("Failed to invoke firewall CLI command, rc: %d, err: %s", rc, err,
                 error_code=ec.OPERATION_FAILED)
        out = None

    return out

# commands

class FirewallShowOvsRules(CliCommandProvider):
    def __call__(self, cmd_list):
        cmd_result = ""
        subcmd_list = ["dfw/ovs-rules"]
        res = get_firewall_cli_output(subcmd_list)
        if res is None:
            cmd_result = "MSG_FIREWALL_KVM_COMM_ERROR"
        else:
            cmd_result = "OK\n\n%s" % res

        return cmd_result


class FirewallShowDpctl(CliCommandProvider):
    def __call__(self, cmd_list):
        cmd_result = ""
        ctl_cmd_list = ["/usr/bin/ovs-dpctl"]
        arg_list = [cmd_list[3]]
        if len(cmd_list) > 4:
           params = cmd_list[4]
           params_list = params.split(" ")
           arg_list.extend(params_list)
        ctl_cmd_list.extend(arg_list)
        rc, out, err = invoke_command(ctl_cmd_list)
        # show errors as well
        out2 = { "results" : "%s" % out }
        cmd_result = "OK\n\n%s" % json.dumps(out2)

        return cmd_result

class FirewallShowDpctlNoArg(CliCommandProvider):
    def __call__(self, cmd_list):
       cmd_result = ""
       ctl_cmd_list = ["/usr/bin/ovs-dpctl"]
       ctl_cmd_list.extend([cmd_list[3]])
       rc, out, err = invoke_command(ctl_cmd_list)
       # show errors as well
       out2 = { "results" : "%s" % out }
       cmd_result = "OK\n\n%s" % json.dumps(out2)

       return cmd_result

class FirewallShowVsctl(CliCommandProvider):
    show_interface = False
    def __call__(self, cmd_list):
       cmd_result = ""
       ctl_cmd_list = ["/usr/bin/ovs-vsctl"]
       arg_list = [cmd_list[3]]
       if self.show_interface is True:
          arg_list.extend([cmd_list[4]])
       ctl_cmd_list.extend(arg_list)
       rc, out, err = invoke_command(ctl_cmd_list)
       # show errors as well
       out2 = { "results" : "%s" % out }
       cmd_result = "OK\n\n%s" % json.dumps(out2)

       return cmd_result

class FirewallShowVsctlIf(FirewallShowVsctl):
    show_interface = True


class FirewallShowProfileFqdn(CliCommandProvider):
    def __call__(self, cmd_list):
        cmd_result = ""
        subcmd_list = ["dfw/fqdn"]
        subcmd_list.extend([cmd_list[2]])
        subcmd_list.extend(["-json"])
        res = get_firewall_cli_output(subcmd_list)
        if res is None:
            cmd_result = "MSG_FIREWALL_KVM_COMM_ERROR"
        else:
            resjson = json.loads(res)
            data = resjson.get("results")
            cmd_result = "OK\n\n%s" % data

        return cmd_result
