# Copyright (C) 2021 VMware, Inc. All rights reserved.
# -- VMware Confidential
import collections

from sha.contrib.action.abc_base import Action
from sha.preconfig import get_sub_logger

from sha.core.whiteboard._data_whiteboard import DataWhiteboard

from sha.core.rmdy._rmdy_center import RmdyCenter

_logger = get_sub_logger(__name__, s2comp="remedy-action")


class Remedy(Action):
    """
    Action for remediation. It passes remedy request to remedy center, return result to plugin script and report
    result to whiteboard.
    """

    @classmethod
    def do(cls, hubs, *args):
        def kernel(**kwargs):
            if not kwargs["cmd"] or not kwargs["plugin_id"]:
                raise Exception("Remedy - Invalid cmd:{}".format(kwargs["cmd"]))
            if not kwargs["plugin_id"]:
                raise Exception("Remedy - Invalid plugin id:{}".format(kwargs["plugin_id"]))

            res = RmdyCenter.rmdy(kwargs["cmd"], kwargs["plugin_id"])
            _logger.debug('Remedy Action - send cmd: {} to rmdy center and get result:{}'.format(kwargs["cmd"], res))
            cls.record_action_result(kwargs["cmd"], kwargs["plugin_id"], res)
            return res

        return kernel

    @classmethod
    def record_action_result(cls, cmd, plugin_id, res):
        name = cls.create_metric_name(plugin_id, cmd)
        metrics = DataWhiteboard.get_metrics(plugin_id, [name])
        if metrics is None:
            # some error happened in whiteboard fetching metrics
            _logger.warning('Remedy Action - Failed in getting metrics from whiteboard, '
                            'metric_name:{}, plugin_id:{}'.format(name, plugin_id))
            value = collections.deque(maxlen=3)
        elif metrics[name] is None:
            # not reported result before
            value = collections.deque(maxlen=3)
        else:
            value = metrics[name]['value']

        value.append(res)
        DataWhiteboard.report_data(name, value)

    @staticmethod
    def create_metric_name(plugin_id, cmd):
        return 'nsx.sha.plugin-id-{}.action-result.rmdy.cmd-{}'.format(plugin_id, cmd)

    @staticmethod
    def create_prefix_metric_name(plugin_id):
        return 'nsx.sha.plugin-id-{}.action-result.rmdy'.format(plugin_id)
